#include <STC8051U.H>
#include "stdio.h"
#include <intrins.h>
#include "assert.h"
#include "peripherals.h"
#include "main.h"


void peripherals_init(void)
{
	clock_config();
	gpio_init();
	delay_init();
	btns_init();
	PWMA_init();
	uart1_init();
}

void clock_config(void)
{
	EA = 0;
	CKCON = 0x00;			      // 设置外部数据总线为最快
	WTST = 0;               	// 设置程序代码等待参数，赋值为0可将CPU执行程序的速度设置为最快
	P_SW2 = 0x80;			    // 开启特殊地址访问

	USBCLK |= (1<<7);		//打开PLL功能
	USBCLK &= ~(3<<5);
	USBCLK |= (2<<5);
	NOP(5);					//等待时钟稳定
	CLKSEL |= (1<<7);
	CLKSEL |= (1<<6);		//PLL->高速IO
	MCLKOCR = 0x01;
	IRC48MCR = 0x80;		//启动内部48M USB时钟
	while(!(IRC48MCR&1));
	IRCBAND &= ~(3<<6);
	IRCBAND |= (2<<6);
	EA = 1;
}

void gpio_init(void)
{
	P0M0 = 0x00; P0M1 = 0x00; 
	P1M0 = 0x00; P1M1 = 0x00; 
	P2M0 = 0x00; P2M1 = 0x00;
	P3M0 = 0x00; P3M1 = 0x00;
	P4M0 = 0x00; P4M1 = 0x00;
}	

#define timer_read() ((unsigned int)PWMB_CNTRH << 8 | PWMB_CNTRL)

void delay_init(void)
{
    TM0PS = 39;
	AUXR |= 0x80;			//定时器时钟1T模式
	TMOD &= 0xF0;			//设置定时器模式
	TMOD |= 0x01;			//设置定时器模式
	TL0 = 0x00;				//设置定时初始值
	TH0 = 0x00;				//设置定时初始值
	TF0 = 0;				//清除TF0标志
	TR0 = 1;				//定时器0开始计时
}

void timer_disable(void) 
{
	PWMB_CR1 &= ~0x01;
}

void timer_enable(void) 
{
	PWMB_CR1 |= 0x01;
}

void delay_us(unsigned int us)
{
    TH0 = 0x00;
    TL0 = 0x00;
    while (((uint16_t)TH0 << 8 | TL0) < us);
}

void delay_ms(unsigned int ms)
{
    uint32_t i;
    for(i = 0;i < ms; i++){
        TH0 = 0x00;
        TL0 = 0x00;
        while (((uint16_t)TH0 << 8 | TL0) < 1000);
    }
}

#include "multi_button.h"

uint8_t xdata btns1_id = 0;
uint8_t xdata btns5_id = 1;
uint8_t xdata btns9_id = 2;
uint8_t xdata btns17_id = 3;

struct Button xdata btns1;
struct Button xdata btns5;
struct Button xdata btns9;
struct Button xdata btns17;

uint8_t read_btns1(uint8_t id)
{
	return P17;
}

uint8_t read_btns5(uint8_t id)
{
	return P16;
}

uint8_t read_btns9(uint8_t id)
{
	return P15;
}

uint8_t read_btns17(uint8_t id)
{
	return P14;
}

extern int16_t duty;
extern void btns1_handler(void *btn);
extern void btns5_handler(void *btn);
extern void btns9_handler(void *btn);
extern void btns17_handler(void *btn);

void Timer11_Init(void)		//5毫秒@45MHz
{
	P_SW2 |= 0x80;			//使能访问XFR寄存器
	T11CR &= 0xf3;			//选择时钟源 (系统时钟SYSclk)
	T11CR &= 0xEF;			//定时器时钟12T模式
	T11L = 0xE5;			//设置定时初始值
	T11H = 0xBE;			//设置定时初始值
	T11CR |= 0x80;			//定时器11开始计时
	T11CR |= 0x02;			//使能定时器11中断
}

void btns_init(void)
{	
	P14 = P15 = P16 = P17 = 1;
	P13 = 0;

	button_init(&btns1, read_btns1, 0, btns1_id);
	button_attach(&btns1, PRESS_DOWN, btns1_handler);
	button_attach(&btns1, LONG_PRESS_HOLD, btns1_handler);
	button_start(&btns1);

	button_init(&btns5, read_btns5, 0, btns5_id);
	button_attach(&btns5, PRESS_DOWN, btns5_handler);
	button_attach(&btns5, LONG_PRESS_HOLD, btns5_handler);
	button_start(&btns5);

	button_init(&btns9, read_btns9, 0, btns9_id);
	button_attach(&btns9, PRESS_DOWN, btns9_handler);
	button_attach(&btns9, LONG_PRESS_HOLD, btns9_handler);
	button_start(&btns9);

	button_init(&btns17, read_btns17, 0, btns17_id);
	button_attach(&btns17, PRESS_DOWN, btns17_handler);
	button_attach(&btns17, LONG_PRESS_HOLD, btns17_handler);
	button_start(&btns17);

	Timer11_Init();
}

void DMA_M2M(uint32_t src_addr,uint32_t aim_addr,uint16_t length)
{
	DMA_M2M_CFG = 0x80;
	DMA_M2M_CR = 0x80;
	DMA_M2M_AMTH = (length - 1) >> 8;
	DMA_M2M_AMT = (length - 1) & 0xFF;
	DMA_M2M_TXAH = src_addr >> 8;
	DMA_M2M_TXAL = src_addr & 0xFF;
	DMA_M2M_RXAH = aim_addr >> 8;
	DMA_M2M_RXAL = aim_addr & 0xFF;
}

void change_duty(void)
{
	// PWMA_CCR1H = duty >> 8;
	// PWMA_CCR1L = duty & 0xFF;
	// PWMA_CCR2H = duty >> 8;
	// PWMA_CCR2L = duty & 0xFF;
	// PWMA_CCR3H = duty >> 8;
	// PWMA_CCR3L = duty & 0xFF;
	// PWMA_CCR4H = duty >> 8;
	// PWMA_CCR4L = duty & 0xFF;
}

void Timer11_Isr(void) INTERRUPT(TMR11_VECTOR) 
{
	button_ticks();
	change_duty();
}

void PWMA_init(void)
{
	//PS
	// PWMA_PS = 0xAA;		//PWMA所有引脚切换到P2端口
	// PWMA_ENO = 0xFF; 	//启用全部通道的PWM输出
	// PWMA_IOAUX = 0x00;

	PWMB_PSCRH = 0x00;	//PWM时钟选择
	PWMB_PSCRL = 0x00;	//PWM时钟选择
	PWMB_BKR = 0xC0;
	PWMB_CCER1 = 0x00;
	// BWMA_SMCR = 0x40;
	PWMB_CCMR1 = 0x01;
	PWMB_CCER1 = 0x01;
	PWMB_ARRH = 0xFF;
	PWMB_ARRL = 0xFF;
	PWMB_IER |= 0x02;
	// BWMA_CCER1 = 0xDD;
	// BWMA_CCER2 = 0xDD;
	PWMB_EGR = 0x01;	//更新PWM寄存器
	PWMB_CR1 |= 0x01;	//启动计数器
}



void uart1_init(void)	//921600bps@40MHz
{
	SCON = 0x50;		//8位数据,可变波特率
	AUXR |= 0x01;		//串口1选择定时器2为波特率发生器
	AUXR |= 0x04;		//定时器时钟1T模式
	T2L = 0xF5;			//设置定时初始值
	T2H = 0xFF;			//设置定时初始值
	AUXR |= 0x10;		//定时器2开始计时
}

#pragma FUNCTIONS (static)
char putchar(char c)
{
	// serialwriteChar(c);
	SBUF = c;
	while (!TI);
	TI = 0;
	return c;
}

